
// example partially generated by chatGPT

package main

import "fmt"

func main() {
    // --- Array Example ---
    var arr [5]int           // Declare an array of length 5
                             // type is:  [5]int  array of int length 5
    arr[0] = 10              // Assign values to the array
    arr[1] = 20
    fmt.Println("Array:", arr)  // Output: Array: [10 20 0 0 0]

    // Accessing array elements by index
    fmt.Println("First element of array:", arr[0]) // Output: 10

    // Arrays have a fixed size, attempting to go out of bounds causes an error
    // arr[5] = 50  // This would result in a compile-time error
    fmt.Println("Array length is ", len(arr))  // Output: Array length is 5
    arr[4] = 50     // last element is at subscript length-1
    fmt.Println("Array:", arr)  // Output: Array: [10 20 0 0 50]

    // --- Slice Example ---
    sl := make([]int, 3)      // Declare a slice of length 3 using make
    sl[0] = 100               // Assign values to the slice
    sl[1] = 200
    fmt.Println("Slice:", sl) // Output: Slice: [100 200 0]

    // Accessing slice elements by index
    fmt.Println("First element of slice:", sl[0])  // Output: 100

    // Slices are dynamic, you can append more elements
    sl = append(sl, 400, 500) // Append new values to the slice
    fmt.Println("Slice after append:", sl) // Output: 
                                           // Slice after append: [100 200 0 400 500]

    // Slices can also dynamically grow and shrink in size
    fmt.Println("Length of slice:", len(sl))    // Output: 5
    fmt.Println("Capacity of slice:", cap(sl))  // Output: 6 
                                                // ( capacity may be greater than len 
                                                // due to internal optimizations)

    // Slices share the underlying array with the original slice
    slCopy := sl[:3]          // Create a new slice referencing the same underlying array
    fmt.Println("Slice copy:", slCopy)  // Output: [100 200 0]
    slCopy[0] = 999
    fmt.Println("Modified Slice Copy:", slCopy) // Output: [999 200 0]
    fmt.Println("Original Slice after modifying copy:", sl)  // Output: [999 200 0 400 500]


    // lets dynamically grow the slice again with an append and see how much
    // the capacity goes up
    sl = append(sl, 700, 800, 1000) // Append new values to the slice
    fmt.Println("Length of slice:", len(sl))    // Output: 8
    fmt.Println("Capacity of slice:", cap(sl))  // Output: 12
                                                // ( capacity might be doubled
                                                // over the 6 it had before

    // lets dynamically grow it one more time
    sl = append(sl, 1, 2, 3, 4, 5) // Append 5 new values to the slice, this
                                   // exceeds the capacity since it is 13 values
    fmt.Println("Length of slice:", len(sl))    // Output: 13
    fmt.Println("Capacity of slice:", cap(sl))  // Output: 24


    // --- Differences between Arrays and Slices ---
    // 1. Arrays have a fixed length, slices are dynamic and can grow.
    // 2. Arrays cannot be resized, while slices can be resized with append.
    // 3. Slices are refs to underlying arrays, so changes in one slice can affect others.
}


