
package main

import (
	"fmt"
	"sync"
	"time"
)

// to make this work we have to synchronize the actions of the
// goroutines on the shared data
// this version makes a mutex lock and everytime a goroutine
// needs to check and update the message buffer, they
// lock the shared buffer, and then unlock it when they
// have written a message to the buffer

var sharedData string // Shared variable buffer for communication
var mu sync.Mutex     // Mutex to synchronize access to sharedData

const pps = 10

func ping( ) {
   for i := 0; i < pps; i++ { // simulate some ping-pong exchanges
      mu.Lock()
      if sharedData == "pong" { // Wait for pong to be written
         fmt.Println("(",i,") Ping received:", sharedData)
         sharedData = "ping"
         fmt.Println("Ping sent:", sharedData)
      }
      mu.Unlock()
      time.Sleep(500 * time.Millisecond) // Simulate processing time
                                         // this also limits starvation by giving 
                                         // the other goroutine a chance to get
                                         // the buffer
   }
}

func pong( ) {
   for i := 0; i < pps; i++ { // simulate some ping-pong exchanges
      mu.Lock()
      if sharedData == "ping" || sharedData == "" { // Wait for ping to be written 
                                                    // or fine initial state
                                                    // this means "pong" goes first
         fmt.Println("(",i,") Pong received:", sharedData)
         sharedData = "pong"
         fmt.Println("Pong sent: ", sharedData)
      }
      mu.Unlock()
      time.Sleep(500 * time.Millisecond) // Simulate processing time
                                         // this also limits starvation by giving 
                                         // the other goroutine a chance to get
                                         // the buffer
   }
}

func main() {
   // Start the ping and pong goroutines
   go ping()
   go pong()

   // Wait for both goroutines to finish
   time.Sleep(pps * 1.2 * time.Second) // just a guess, not a foolproof approach
                                       // since speed of goroutines may depend
                                       // platform physical parameters (cpu speed)
}


