
package main
import (
   "fmt"
   "time"
)

type Actor struct {  state int  }

// Define messages that can be sent to the actor
type Message struct {
   action string
   value  int
}

// Define a stateful process (actor)
func (a *Actor) behavior(messages chan Message) {
   for msg := range messages {
      switch msg.action {
         case "inc":  fmt.Print("inc(",msg.value,"): ")
                      a.state += msg.value
         case "dec":  fmt.Print("dec(",msg.value,"): ")
                      a.state -= msg.value
         case "dub":  fmt.Print("dub(",msg.value,"): ")
                      a.state *= 2
         case "mult": fmt.Print("mult(",msg.value,"): ")
                      a.state *= msg.value
         case "pow":  {
                         fmt.Print("pow(",msg.value,"): ")
                         k := a.state
                         for i := 1; i<msg.value; i++ { 
                            a.state *= k 
                         }
                      }
         case "set":  fmt.Print("set(",msg.value,"): ")
                      a.state = msg.value
         case "zero": fmt.Print("zero(",msg.value,"): ")
                      a.state = 0
      }
      fmt.Println("obj state after is ", a.state)
   }
}

func main() {
   actor := &Actor{state: 0}
   fmt.Println("obj state starting is ", actor.state)
   messages := make(chan Message)

   go actor.behavior(messages) // Start the actor activity in a goroutine

   // Send messages to the actor
   messages <- Message{"inc", 5}
   messages <- Message{"dec", 2}
   messages <- Message{"dub", 14}
   messages <- Message{"mult", 5}
   messages <- Message{"pow", 3}
   messages <- Message{"inc", -12}
   messages <- Message{"dec", -12}
   messages <- Message{"mult", -2}
   messages <- Message{"set", 27}
   messages <- Message{"zero", -34567}

   // Allow some time for processing before closing the channel
   time.Sleep(1 * time.Second)
   close(messages)
}

