package main
import (
   "fmt"
   "time"
)

// Define an actor type
type Actor struct {
   mbox chan string // Channel for receiving messages
   quit chan struct{} // Channel for stopping the actor
}

// Actor's message processing loop
func (a *Actor) Run() {
   for {
      select {
         case msg := <-a.mbox:
            fmt.Println("Actor received:", msg)

	 case <-a.quit:
            fmt.Println("Actor shutting down.")
            return
      }
   }
}

// Function to send messages to the actor
func (a *Actor) SendMessage(msg string) {  a.mbox <- msg  }

// Function to stop the actor
func (a *Actor) Stop() {  close(a.quit)  }

func main() {
   // Create an actor instance
   actor := &Actor {
      mbox: make(chan string),
      quit:    make(chan struct{}),
   }

   // Start the actor in a separate Goroutine 
   go actor.Run()

   // Send messages
   actor.SendMessage("Hello")
   actor.SendMessage("How are you?")

   // Allow some time for processing
   time.Sleep(time.Second)

   // Stop the actor
   actor.Stop()

   // Allow time for cleanup
   time.Sleep(3 * time.Second)
}
