
package main

import (
   "fmt"
   "time"
)

// Define a Message struct to hold the action and value
type Message struct {
   action string // The action to perform (add, sub, mult, msg, zero)
   value  int    // The value to use in arithmetic actions
   msg    string // Custom message to print for 'msg' action
}

// Define an actor type with a running total
type Actor struct {
   mbox  chan Message // Channel for receiving messages
   quit  chan struct{}// Channel for stopping the actor
   total int          // The running total
}

// Actor's message processing loop
func (a *Actor) Run() {
   for {
      select {
         case msg := <-a.mbox:
	    // Process the message based on the action
            switch msg.action {
               case "add":
                  a.total += msg.value
                  fmt.Printf("Added %d, new total: %d\n", msg.value, a.total)

               case "sub":
                  a.total -= msg.value
                  fmt.Printf("Subtracted %d, new total: %d\n", msg.value, a.total)

               case "mult":
                  a.total *= msg.value
                  fmt.Printf("Multiplied by %d, new total: %d\n", msg.value, a.total)

	       case "tot":
                  fmt.Printf("current total: %d\n", a.total)

	       case "msg":
                  // Print the custom message and the current total
                  fmt.Printf("Message: %s\n", msg.msg)
                  //fmt.Printf("Current total: %d\n", a.total)

               case "zero":
                  // Reset the total to zero
                  a.total = 0
                  fmt.Println("Total reset to zero")

               default:
                  fmt.Println("Unknown action:", msg.action)
         }

	case <-a.quit:
	   fmt.Println("quit received... actor shutting down.")
	   return
      }
   }
}

// Function to send messages to the actor
func (a *Actor) Calc(msg Message) {  a.mbox <- msg  } 

// Function to stop the actor
func (a *Actor) Stop() {  close(a.quit)  }

func main() {
   // Create an actor instance with a running total initialized to 0
   actor := &Actor{
      mbox:  make(chan Message),
      quit:  make(chan struct{}),
      total: 0,
   }

   // Start the actor in a separate Goroutine
   go actor.Run()

   // Send messages to the actor
   actor.Calc(Message{action: "add", value: 5})      // Add 5
   actor.Calc(Message{action: "mult", value: 3})     // Multiply by 3
   actor.Calc(Message{action: "sub", value: 2})      // Subtract 2
   actor.Calc(Message{action: "msg", msg: "hello... middle"}) 
   actor.Calc(Message{action: "tot"})
   actor.Calc(Message{action: "zero"})               // Reset total to zero
   actor.Calc(Message{action: "msg", msg: "After reset."}) 

   // Allow some time for processing
   time.Sleep(2 * time.Second)

   // Stop the actor
   actor.Calc(Message{action: "msg", msg: "main about to call stop"}) 
   actor.Calc(Message{action: "tot"})
   actor.Stop()

   // Allow time for cleanup
   time.Sleep(1 * time.Second)
   fmt.Println("main quits")
}
